package org.processmining.ptconversions.plugins;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.processtree.ProcessTree;
import org.processmining.ptconversions.pn.ProcessTree2Petrinet;
import org.processmining.ptconversions.pn.ProcessTree2Petrinet.InvalidProcessTreeException;
import org.processmining.ptconversions.pn.ProcessTree2Petrinet.NotYetImplementedException;
import org.processmining.ptconversions.pn.ProcessTree2Petrinet.PetrinetWithMarkings;

public class PluginPN {
	@Plugin(name = "Convert Process Tree to Petri Net", returnLabels = { "Petri net", "Initial Marking",
	"Final Marking" }, returnTypes = { Petrinet.class, Marking.class, Marking.class }, parameterLabels = { "Process Tree" }, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "S.J.J. Leemans", email = "s.j.j.leemans@tue.nl")
	@PluginVariant(variantLabel = "Convert Process Tree to Petri Net, default", requiredParameterLabels = { 0 })
	public Object[] convert(PluginContext context, ProcessTree tree) throws NotYetImplementedException,
		InvalidProcessTreeException {
		PetrinetWithMarkings result = ProcessTree2Petrinet.convert(tree, false);
		
		context.addConnection(new InitialMarkingConnection(result.petrinet, result.initialMarking));
		context.addConnection(new FinalMarkingConnection(result.petrinet, result.finalMarking));
		
		return new Object[] { result.petrinet, result.initialMarking, result.finalMarking };
	}
	
	@Plugin(name = "Convert Process Tree to Petri Net - keep structure", returnLabels = { "Petri net",
		"Initial Marking", "Final Marking" }, returnTypes = { Petrinet.class, Marking.class, Marking.class }, parameterLabels = { "Process Tree" }, userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "S.J.J. Leemans", email = "s.j.j.leemans@tue.nl")
	@PluginVariant(variantLabel = "Convert Process Tree to Petri Net, default", requiredParameterLabels = { 0 })
	public Object[] convertKeepStructure(PluginContext context, ProcessTree tree) throws NotYetImplementedException,
		InvalidProcessTreeException {
		PetrinetWithMarkings result = ProcessTree2Petrinet.convert(tree, true);
		
		context.addConnection(new InitialMarkingConnection(result.petrinet, result.initialMarking));
		context.addConnection(new FinalMarkingConnection(result.petrinet, result.finalMarking));
		
		return new Object[] { result.petrinet, result.initialMarking, result.finalMarking };
	}
}
