package org.processmining.ptconversions.yawl;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.processtree.Originator;
import org.processmining.processtree.ProcessTree;

@Plugin(
	name = "Convert Process Tree to YAWLOrgData", 
	parameterLabels = { "Process Tree" }, 
	returnLabels = { "YAWL Org Data" }, 
	returnTypes = { YAWLOrgData.class }, 
	userAccessible = true)
public class YAWLOrgData {
	
	private static String YAWLHash = "Se4tMaQCi9gr0Q2usp7P56Sk5vM="; // the hash of "YAWL" as password :-)
	private String XMLvalue;
	
	public YAWLOrgData(){
		
	}
	
	public String getXML(){
		return XMLvalue;
	}
	
	public void setXML(String XML){
		XMLvalue = XML;
	}
	
	/**
	 * We just generate the XML, seems the easiest to do
	 * @param tree
	 * @return
	 */
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "D.M.M. Schunselaar", email = "d.m.m.schunselaar@tue.nl", uiLabel = "Convert ProcessTree to YAWLOrgData")
	@PluginVariant(variantLabel = "Select Tree to convert", requiredParameterLabels = { 0 })
	public YAWLOrgData PT2YBKPXML(PluginContext context, ProcessTree tree){
		YAWLOrgData orgData = new YAWLOrgData();
		String XML = "";
		XML += "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		XML += "<orgdata>\n";
		XML += "\t<participants>\n";
		for(Originator org: tree.getOriginators()){
			if(org instanceof Originator.Role){
				XML += writePariticipant((Originator.Role)org);
			}
		}
		XML += "\t</participants>\n";
		XML += "\t<roles>\n";
		for(Originator org: tree.getOriginators()){
			if(org instanceof Originator.Role){
				XML += writeRole((Originator.Role)org);
			}
		}
		XML += "\t</roles>\n";
		XML += "\t<positions/>\n";
		XML += "\t<capabilities/>\n";
		XML += "\t<orggroups>\n";
		for(Originator org: tree.getOriginators()){
			if(org instanceof Originator.Group){
				XML += writeGroup((Originator.Group)org);
			}
		}
		XML += "\t</orggroups>\n";
		XML += "\t<nonhumanresources/>\n";
		XML += "\t<nonhumancategories/>\n";
		XML += "</orgdata>";
		orgData.setXML(XML);
		return orgData;
	}
	
	private String writePariticipant(Originator.Role role){
		String indenting = "\t\t";
		String XML = "";
		XML += indenting + "<participant id=\"PA-" + role.getID().toString() + "\">\n";
		XML += indenting + "\t<userid>" + role.getName() + "</userid>\n";
		XML += indenting + "\t<password>" + YAWLHash + "</password>\n";
		XML += indenting + "\t<firstname>" + role.getName() + "</firstname>\n";
		XML += indenting + "\t<lastname>" + role.getName() + "</lastname>\n";
		XML += indenting + "\t<description>Generated so no desciption given</description>\n";
		XML += indenting + "\t<notes/>\n";
		XML += indenting + "\t<isAdministrator>false</isAdministrator>\n";
		XML += indenting + "\t<notes/>\n";
		XML += indenting + "\t<roles>\n";
		XML += indenting + "\t\t<role>RO-" + role.getID().toString() + "</role>\n";
		XML += indenting + "\t</roles>\n";
		XML += indenting + "\t<positions/>\n";
		XML += indenting + "\t<capabilities/>\n";
		XML += indenting + "\t<privileges>00000000</privileges>\n";
		XML += indenting + "</participant>\n";
		return XML;
		
	}
	
	private String writeRole(Originator.Role role){
		String indenting = "\t\t";
		String XML = "";
		XML += indenting + "<role id=\"RO-" + role.getID().toString() + "\">\n";
		XML += indenting + "\t<name>" + role.getName() + "</name>\n";
		XML += indenting + "\t<description/>\n";
		XML += indenting + "\t<notes/>\n";
		XML += indenting + "</role>\n";
		return XML;
	}
	
	private String writeGroup(Originator.Group group){
		String indenting = "\t\t";
		String XML = "";
		XML += indenting + "<orggroup id=\"OG-" + group.getID() + "\">\n";
		XML += indenting + "\t<groupName>" + group.getName() + "</groupName>";
		XML += indenting + "\t<groupType>GROUP</groupType>\n";
		XML += indenting + "\t<description/>\n";
		XML += indenting + "\t<notes/>\n";
		XML += indenting + "</orggroup>\n";
		return XML;
	}
}
